#!/usr/bin/env python
###################################
#
#    check-terracotta-server.py
#
#
###################################
import sys, os, re
import Globals
from optparse import OptionParser
from Products.ZenUtils.ZenScriptBase import ZenScriptBase
from ZenPacks.community.zenJavaApp.modeler.plugins.JavaAppMap import JavaAppScan

class TerracottaStatus(ZenScriptBase):
    '''
        Check to test Terracotta server status (active or passive) and number of clients connected
    '''
    def __init__(self):
        ZenScriptBase.__init__(self, connect=False)
        self.exitStatus = 0
        self.exitMessage = ''
        self.isPassive = False
        self.problemExists = False
        self.clients = 0
        self.status = "OK"

    def isPassiveStandby(self):
        """ whether this server is a passive standby
        """
        mbean = 'org.terracotta.internal:type=Terracotta Server,name=Terracotta Server'
        attribute = 'PassiveStandby'
        lines = self.scan.jmxQuery(self.options.tcpport, self.auth, mbean, attribute)
        for line in lines:
            if re.search('true',line) != None :
                self.isPassive =  True
                
    def numClients(self):
        """ find number of currently connected clients
        """
        mbean = 'org.terracotta:type=Terracotta Server,name=DSO'
        attribute = 'ClientLiveObjectCount'
        splitter = 'org.terracotta:name=DSO,type=Terracotta Server,channelID'
        lines = self.scan.jmxQuery(self.options.tcpport, self.auth, mbean, attribute)
        output = lines[0]
        attr,value = output.split(attribute+': {')
        self.clients = len(value.split(splitter)) - 1

    def quit(self):
        """
        exit out Nagios-style
        """
        if self.problemExists == True:
            self.exitMessage = "%s: Server check failed | clients=%s" % (self.status, self.clients)
            self.exitStatus = 2
        else:
            self.exitMessage = "%s: Server is %s| clients=%s" % (self.status, self.state, self.clients)
            self.exitStatus = 0
        print self.exitMessage
        sys.exit(self.exitStatus)
        
    def run(self):
        """ main execution statement
        """
        if self.options.auth == True:
            self.auth = "%s:%s" %(self.options.username, self.options.password)
        else:
            self.auth = ''
        #self.port = self.options.port
        self.scan = JavaAppScan(self.options.server, self.options.server, '', self.options.username, self.options.password)
        self.isPassiveStandby()
        if self.isPassive == True:
            self.state = "Passive Standby"
            self.clients = 0
        else:
            self.state = "Active Primary"
            try:
                self.numClients()
            except:
                self.problemExists = True
                self.status = "WARNING"
        self.quit()
        
    def buildOptions(self):
        ZenScriptBase.buildOptions(self)
        self.parser.add_option('-s','--server', dest='server',
            help='Remote server host')
        self.parser.add_option('-p','--tcpport', dest='tcpport',
            help='JMX Port')
        self.parser.add_option('-a','--auth', dest='auth',
            help='JMX Authentication', action="store_true")
        self.parser.add_option('-u','--username', dest='username',
            help='User Name')
        self.parser.add_option('-w','--password', dest='password',
            help='Password')
        
if __name__ == "__main__":
    u = TerracottaStatus()
    u.run()
