from Products.DataCollector.plugins.CollectorPlugin import CollectorPlugin
from Products.DataCollector.plugins.DataMaps import ObjectMap
from ZenPacks.community.zenJavaApp.modeler.plugins.JavaAppMap import JavaAppScan
from ZenPacks.community.zenTerracotta.Definition import *
from Products.ZenUtils.Utils import zenPath,prepId
import re,os,time,random, socket


class TerracottaClientMap(CollectorPlugin):
    """Map JMX Client output table to model."""
    
    compname = "os"
    constr = Construct(TerracottaClientDefinition)
    relname = constr.relname
    modname = constr.zenpackComponentModule
    baseid = constr.baseid
    transport = "python"
    
    deviceProperties = CollectorPlugin.deviceProperties + (
                    'zJmxUsername',
                    'zJmxPassword',
                    'zJavaAppPortRange'
                    'manageIp',
                    )

    def testTerracotta(self, port, auth, log):
        """ 
            look for terracotta-related mbeans
        """
        mbean = 'org.terracotta:type=Terracotta Server,name=DSO'
        attribute = 'ClientLiveObjectCount'
        try:
            lines = self.scan.jmxQuery(port, auth, mbean, attribute)
            #for line in lines:  print "  %s"  % line
            if attribute in lines[0]:
                return True
            else:
                return False
            return True
        except:
            return False
    
    def clientInfo(self):
        info = {
                'id': '',
                'server': '',
                'serverport': 9520,
                'serveruser': '',
                'serverpassword': '',
                'serverauth': False,
                'node': '',
                'nodeport': '',
                'nodeid': '',
                'nodeip': '',
                'nodeuser': '',
                'nodepassword': '',
                'nodeauth': False,
                'nodeavailable': False,              
                }
        return info
    
    def findPort(self, lines):
        finder = '-rmiRegistryPort'
        for line in lines:
            if finder in line:
                return line[line.find(finder):].split(' ')[1]
        return None
   
    def getClientInfo(self,port,auth,log):
        """"""
        output = []
        lines = self.scan.jmxQuery(port, auth)
        #log.debug("lines: %s" % lines)
        findstring = 'org.terracotta:clients=Clients,name=L1 Info Bean'
        splitter = 'node='
        for line in lines:
            #print line
            if re.search(findstring, line) != None:
                # pull client info from terracotta server
                clientport = None
                path, nodeinfo = line.split(splitter)
                client, nodeid = nodeinfo.split('/')
                nodeid,nodetype = nodeid.split(',')
                mbean = '%snode=%s/%s,%s' % (path, client, nodeid, nodetype)
                clines = self.scan.jmxQuery(port, auth, mbean, 'Environment')
                # try to get the client's JMX port
                clientport = self.findPort(clines)
                # now we have valid client port info
                if clientport != None:
                    #log.debug("server: %s port: %s active" % (server,cport))
                    info = self.clientInfo()
                    clientscan = JavaAppScan(client, client, '', self.scan.username, self.scan.password)
                    clientscan.testPort(clientport)
                    name = "%s_%s_%s" % (self.baseid,client,str(clientport))
                    
                    info['id'] = prepId(name)
                    info['server'] = self.scan.device
                    info['serverport'] = port
                    info['serveruser'] = self.scan.username
                    info['serverpassword'] = self.scan.password
                    info['serverauth'] = self.scan.portdict[port]['useAuth']
                    
                    info['node'] = client
                    info['nodeport'] = clientport
                    info['nodeid'] = nodeid
                    info['nodeip'] = socket.gethostbyname(client)
                    info['nodeauth'] = False
                    info['nodeuser'] = self.scan.username
                    info['nodepassword'] = self.scan.password
                    info['nodeavailable'] = False
                    # now we need to test each client port for JMX connectivity and the connection status mbean
                    
                    cportinfo = clientscan.portdict[clientport]
                    info['nodeauth'] = cportinfo['useAuth']
                    
                    print cportinfo
                    if cportinfo['isJmx'] == True:
                        authstring = ''
                        if cportinfo ['useAuth'] == True:
                            if cportinfo['validAuth'] == True:
                                authstring = "%s:%s" %(self.scan.username, self.scan.password)
                        # lastly check for the presence of the client connected status mbean
                        mbean = 'atxg:component-name=AppServer,subcomponent-name=EhCacheManager,mbean-name=TerracottaClient'
                        clientcheck = clientscan.jmxQuery(clientport, authstring, mbean, 'CacheManagerStatusValue')[0]
                        #print "CLIENT CHECK: %s" % clientcheck
                        if len(clientcheck) >0 :
                            if 'CacheManagerStatusValue' in clientcheck[0]:
                                info['nodeavailable'] = True
                    output.append(info)
                    print info
        return output
                        
    def collect(self, device, log):
        self.scan = JavaAppScan(device.id, device.manageIp, device.zJavaAppPortRange, device.zJmxUsername, device.zJmxPassword)
        self.scan.evalPorts()
        for port in self.scan.portdict.keys():
            entry = self.scan.portdict[port]
            auth = ''
            # first identify possible ports
            if entry['isJmx'] == True:
                if entry['useAuth'] == True:
                    if entry['validAuth'] == True: auth = "%s:%s" %(device.zJmxUsername, device.zJmxPassword)
                # if port is a terracotta server, get client info
                if self.testTerracotta(port, auth, log) == True:
                    #print "testing Terracotta Server :%s" % port
                    return self.getClientInfo(port,auth,log)

    def process(self, device, results, log):
        log.info("results: %s" % results)
        rm = self.relMap()
        for result in results:
            om = self.objectMap(result)
            #print om
            om.monitor = result['nodeavailable']
            om.setTerracottadevice = 'a'
            om.setTerracottaserver = 'a'
            om.setTerracottajavaapp = 'a'
            rm.append(om)
        return rm
    

