######################################################################
#
# TerracottaServerMap modeler plugin
#
######################################################################
from Products.DataCollector.plugins.CollectorPlugin import CollectorPlugin
from Products.DataCollector.plugins.DataMaps import ObjectMap
from ZenPacks.community.zenJavaApp.JavaAppScan import *
from ZenPacks.community.zenTerracotta.Definition import *
from Products.ZenUtils.Utils import prepId
import re

class TerracottaServerMap(CollectorPlugin):
    """Map JMX Client output table to model."""

    constr = Construct(TerracottaServerDefinition)
    
    compname = "os"
    relname = constr.relname
    modname = constr.zenpackComponentModule
    baseid = constr.baseid
    transport = "python"
    clients = 0
    
    deviceProperties = CollectorPlugin.deviceProperties + tuple([p[0] for p in TerracottaServerDefinition.packZProperties]) + (
                    'zJmxUsername',
                    'zJmxPassword',
                    'manageIp',
                    )

    def testTerracotta(self, port, auth, log):
        """ 
            look for terracotta-related mbeans
        """
        mbean = 'org.terracotta:type=Terracotta Server,name=DSO'
        attribute = 'ClientLiveObjectCount'
        splitter = 'org.terracotta:name=DSO,type=Terracotta Server,channelID'
        try:
            lines = self.scan.jmxQuery(port, auth, mbean, attribute)
            if attribute in lines[0]:
                output = lines[0]
                attr,value = output.split(attribute+': {')
                self.clients = len(value.split(splitter)) - 1
                return True
            else:
                return False
        except:
            return False
    
    def collect(self, device, log):
        self.scan = JavaAppScan(device.id, device.manageIp, 
                                device.zTerracottaServerPortRange, 
                                device.zJmxUsername, device.zJmxPassword)
        self.scan.evalPorts()
        output = []
        for port in self.scan.portdict.keys():
            entry = self.scan.portdict[port]
            if entry['isJmx'] == True:
                
                name = "%s_%s" % (self.baseid,str(port))
                info = {'id' : prepId(name),
                        'server' : device.id,
                        'port' : port,
                        'authenticate' : entry['useAuth'],
                        'user' : '',
                        'password' : '',
                        'isWorking' : False,
                        }
                if entry['useAuth'] == False: # authentication isn't needed
                    info['isWorking'] = True
                else:
                    if entry['validAuth'] == True: # we can use the zJmxPassword
                        info['user'] = device.zJmxUsername
                        info['password'] = device.zJmxPassword
                        info['isWorking'] = True
                    else:
                        info['isWorking'] = False # we don't know the username/password
                if info['isWorking'] == True:
                    auth = "%s:%s" %(device.zJmxUsername, device.zJmxPassword)
                    if self.testTerracotta(port, auth, log) == True:
                        output.append(info)
        return output
    
    def process(self, device, results, log):
        log.info("results: %s" % results)
        rm = self.relMap()
        for result in results:
            om = self.objectMap(result)
            om.clients = self.clients
            om.setTerracottaport = om.port
            om.monitor = result['isWorking']
            rm.append(om)
            print om
        return rm
